@@ -2,12 +2,13 @@
 ## @brief Common code to odfpy and openpyxl wrappers
 from datetime import datetime, date
 from gettext import translation
-from pkg_resources import resource_filename
 from logging import info, ERROR, WARNING, INFO, DEBUG, CRITICAL, basicConfig, error
+from pkg_resources import resource_filename
+from psutil import process_iter
 from uno import createUnoStruct
 
-__version__ = '0.5.0'
-__versiondatetime__=datetime(2021, 10, 3, 16, 51)
+__version__ = '0.8.0'
+__versiondatetime__=datetime(2021, 10, 18, 17, 41)
 __versiondate__=__versiondatetime__.date()
 
 try:
@@ -112,6 +113,9 @@ class Coord:
         self.letter, self.number=self.__extract(strcoord)
 
     def __repr__(self):
+        return f"Coord <{self}>"
+        
+    def __str__(self):
         return self.string()
 
     def __eq__(self, b):
@@ -204,8 +208,11 @@ class Coord:
 class Range:
     def __init__(self,strrange):
         self.start, self.end=self.__extract(strrange)
-        
+
     def __repr__(self):
+        return f"Range <{self}>"
+        
+    def __str__(self):
         return self.string()
 
     ##Return the outcome of the test b in a. Note the reversed operands.
@@ -351,6 +358,12 @@ def Coord_from_letters(column, letter):
 ## Creates a Coord object from spreadsheet index coords
 def Coord_from_index(column_index, row_index):
     return Coord(index2column(column_index)+index2row(row_index))
+    
+## Creates a Range object from itself stard and end coords
+def Range_from_coords(start, end):
+    start=Coord.assertCoord(start)
+    end=Coord.assertCoord(end)
+    return Range(f"{start}:{end}")
 
 def generate_formula_total_string(key, coord_from, coord_to):
     if key == "#SUM":
@@ -406,3 +419,33 @@ def next_port(last,  first_port,  instances):
         return first_port
     else:
         return last+1
+
+def get_range_from_iterable_object( coord_start, o):
+    coord_start=Coord.assertCoord(coord_start)
+    r=Range(f"{coord_start.string()}:{coord_start.string()}")
+
+    if len(o)==0:
+        return r
+
+    if o[0].__class__.__name__ in ("list","dict", "OrderedDict"): #Iterables con len
+        len_rows=len(o)-1
+        len_columns=len(o[0])-1
+    else:
+        len_rows=0
+        len_columns=len(o)-1
+    return Range(f"{coord_start.string()}:{coord_start.addRowCopy(len_rows).addColumnCopy(len_columns).string()}")
+
+def get_from_process_numinstances_and_firstport():        
+    try:
+        instances=0
+        ports=[]
+        for p in process_iter(['name','cmdline', 'pid']): 
+            if p.info['name']=='soffice.bin':
+                if  'file:///tmp/unogenerator'  in ' '.join(p.info['cmdline']):
+                    instances=instances+1
+                    ports.append(int(p.info['cmdline'][1][-4:]))
+        first_port=min(ports)
+        return instances, first_port
+    except:
+        print(_("Have you launched unogenerator instances?. Please run unogenerator_start"))
+        return None, None
