@@ -6,15 +6,15 @@ import argparse
 from collections import OrderedDict
 from concurrent.futures import ProcessPoolExecutor, as_completed
 from datetime import datetime, date, timedelta
-from gettext import translation, install
+from gettext import translation
 from multiprocessing import cpu_count
 from pkg_resources import resource_filename
 
-from unogenerator.commons import __version__, addDebugSystem, argparse_epilog, ColorsNamed, Coord as C, next_port
+from unogenerator.commons import __version__, addDebugSystem, argparse_epilog, ColorsNamed, Coord as C, next_port, get_from_process_numinstances_and_firstport
 from unogenerator.reusing.currency import Currency
 from unogenerator.reusing.percentage import Percentage
 from unogenerator.unogenerator import ODT_Standard, ODS_Standard
-from unogenerator.helpers import helper_title_values_total_row,helper_title_values_total_column, helper_totals_row, helper_totals_column, helper_totals_from_range, helper_list_of_ordereddicts, helper_list_of_dicts
+from unogenerator.helpers import helper_title_values_total_row,helper_title_values_total_column, helper_totals_row, helper_totals_column, helper_totals_from_range, helper_list_of_ordereddicts, helper_list_of_dicts, helper_list_of_ordereddicts_with_totals
 from os import remove
 from tqdm import tqdm
 
@@ -36,7 +36,7 @@ def remove_without_errors(filename):
 def main(arguments=None):
     parser=argparse.ArgumentParser(prog='unogenerator', description=_('Create example files using unogenerator module'), epilog=argparse_epilog(), formatter_class=argparse.RawTextHelpFormatter)
     parser.add_argument('--version', action='version', version=__version__)
-    parser.add_argument('--debug', help="Debug program information", choices=["DEBUG","INFO","WARNING","ERROR","CRITICAL"], default="ERROR")
+    parser.add_argument('--debug', help=_("Debug program information"), choices=["DEBUG","INFO","WARNING","ERROR","CRITICAL"], default="ERROR")
     group= parser.add_mutually_exclusive_group(required=True)
     group.add_argument('--create', help="Create demo files", action="store_true",default=False)
     group.add_argument('--remove', help="Remove demo files", action="store_true", default=False)
@@ -57,6 +57,7 @@ def main(arguments=None):
     if args.create==True:
         start=datetime.now()
         futures=[]
+        
         with ProcessPoolExecutor(max_workers=cpu_count()+1) as executor:
             for language in ['es', 'en']:
                 futures.append(executor.submit(demo_ods_standard, language))
@@ -64,7 +65,7 @@ def main(arguments=None):
 
         for future in as_completed(futures):
             future.result()
-        print("All process took {}".format(datetime.now()-start))
+        print(_("All process took {}".format(datetime.now()-start)))
 
 
 def main_concurrent(arguments=None):
@@ -74,15 +75,15 @@ def main_concurrent(arguments=None):
     group= parser.add_mutually_exclusive_group(required=True)
     group.add_argument('--create', help="Create demo files", action="store_true",default=False)
     group.add_argument('--remove', help="Remove demo files", action="store_true", default=False)
-    parser.add_argument('--workers', help="Workers max 8", action="store", default=8,  type=int)
+    parser.add_argument('--workers', help=_("Number of workers to process this script. Default 4"), action="store", default=4,  type=int)
+    parser.add_argument('--loops', help="Loops of documentation jobs", action="store", default=30,  type=int)
     args=parser.parse_args(arguments)
 
-
-    print("You need to launch 8 instances from port 2002 to run this concurrent demo. You can use bash unogenerator_start")
+    num_instances, first_port=get_from_process_numinstances_and_firstport()
     addDebugSystem(args.debug)
 
     if args.remove==True:
-            for i in range(10):
+            for i in range(args.loops):
                 remove_without_errors(f"unogenerator_documentation_en.{i}.odt")
                 remove_without_errors(f"unogenerator_documentation_en.{i}.docx")
                 remove_without_errors(f"unogenerator_documentation_en.{i}.pdf")
@@ -91,17 +92,19 @@ def main_concurrent(arguments=None):
                 remove_without_errors(f"unogenerator_example_en.{i}.pdf")
 
     if args.create==True:
+        print(_(f"Launching concurrent demo with {args.workers} workers to a daemon with {num_instances} instances from {first_port} port"))
+
         start=datetime.now()
         futures=[]
-        port=2002
+        port=first_port
         with ProcessPoolExecutor(max_workers=args.workers) as executor:
-            with tqdm(total=20) as progress:
-                for i in range(10):
-                    port=next_port(port, 2002, 8)
+            with tqdm(total=args.loops*2) as progress:
+                for i in range(args.loops):
+                    port=next_port(port, first_port, num_instances)
                     future=executor.submit(demo_ods_standard, 'en', port, f".{i}")
                     future.add_done_callback(lambda p: progress.update())
                     futures.append(future)
-                    port=next_port(port, 2002, 8)
+                    port=next_port(port, first_port, num_instances)
                     future=executor.submit(demo_odt_standard, 'en', port, f".{i}")
                     future.add_done_callback(lambda p: progress.update())
                     futures.append(future)
@@ -115,15 +118,17 @@ def main_concurrent(arguments=None):
         for future in futures:
             result = future.result()
             results.append(result)
-        print("All process took {}".format(datetime.now()-start))
+        print(_("All process took {}".format(datetime.now()-start)))
 
        
 def demo_ods_standard(language, port=2002, suffix="",):
     if language=="en":
-        lang1=install('unogenerator', 'badlocale')
-    else:
-        lang1=translation('unogenerator', 'unogenerator/locale', languages=[language])
+        lang1=translation('unogenerator' , resource_filename("unogenerator","locale"), languages=[language])
         lang1.install()
+    else:
+        lang1=translation('unogenerator', resource_filename("unogenerator","locale"), languages=[language])
+        lang1.install()
+    ##print(_("My language is "),language,lang1)
     
     doc=ODS_Standard(port)
     doc.setMetadata(
@@ -195,8 +200,8 @@ def demo_ods_standard(language, port=2002, suffix="",):
 
 
     doc.addCellMergedWithStyle("A19:D19","List of rows with totals", ColorsNamed.Orange, "BoldCenter")
-    doc.addListOfRowsWithStyle("A20", [["A",1,2,3],["B",4,5,6],["C",7,8,9]], ColorsNamed.White)
-    helper_totals_from_range(doc, "B20:D22")
+    doc.addListOfRowsWithStyle("A20", [["A",12000,2,3],["B",1020,5,6],["C",20404,8,9]], ColorsNamed.White)
+    helper_totals_from_range(doc, "B20:D22", totalcolumns=True, totalrows=True)
 
     
     doc.addCellMergedWithStyle("A25:B25","List of ordered dictionaries", ColorsNamed.Orange, "BoldCenter")
@@ -207,6 +212,12 @@ def demo_ods_standard(language, port=2002, suffix="",):
     
     doc.addCellMergedWithStyle("A30:B30","List of dictionaries", ColorsNamed.Orange, "BoldCenter")
     helper_list_of_dicts(doc, "A31",  lod, keys=["Song",  "Singer"])
+    
+    doc.addCellMergedWithStyle("A35:D35","List of ordered dictionaries one method with totals", ColorsNamed.Orange, "BoldCenter")
+    lod=[]
+    lod.append(OrderedDict({"Singer": "Elvis",  "Songs": 10000 , "Albums": 100}))
+    lod.append(OrderedDict({"Singer": "Roy Orbison",  "Songs": 100,  "Albums": 20 }))
+    helper_list_of_ordereddicts_with_totals(doc, "A36",  lod, columns_header=1)
 
     doc.removeSheet(0)
     doc.save(f"unogenerator_example_{language}{suffix}.ods")
@@ -214,16 +225,17 @@ def demo_ods_standard(language, port=2002, suffix="",):
     doc.export_pdf(f"unogenerator_example_{language}{suffix}.pdf")
     doc.close()
     
-    r= f"unogenerator_example_{language}{suffix}.ods took {datetime.now()-doc.init} in {port}"
+    r= _(f"unogenerator_example_{language}{suffix}.ods took {datetime.now()-doc.init} in {port}")
     print(r)
     return r
     
     
 def demo_odt_standard(language, port=2002, suffix=""):
     if language=="en":
-        lang1=install('unogenerator', 'badlocale')
+        lang1=translation('unogenerator', resource_filename("unogenerator","locale"), languages=[language])
+        lang1.install()
     else:
-        lang1=translation('unogenerator', 'unogenerator/locale', languages=[language])
+        lang1=translation('unogenerator', resource_filename("unogenerator","locale"), languages=[language])
         lang1.install()
 
     doc=ODT_Standard(port)
@@ -258,7 +270,7 @@ def demo_odt_standard(language, port=2002, suffix=""):
     doc.addParagraph(_("Installation"), "Heading 2")
     doc.addParagraph(_("You can use pip to install this python package:") ,  "Standard")
     doc.addParagraph("""pip install unogenerator"""    , "Code")
-    doc.addParagraph(_("Hello World example"), "Heading 2")
+    doc.addParagraph(_("ODT 'Hello World' example"), "Heading 2")
     doc.addParagraph(_("This is a Hello World example. You get the example in odt, docx and pdf formats:") ,  "Standard")
     doc.addParagraph("""from unogenerator import ODT_Standard
 doc=ODT_Standard()
@@ -285,17 +297,16 @@ doc.close()"""    , "Code")
         _("ODT with standard template (Recomended).") + " " + 
         _("There is a predefined template in code called 'standard.odt', inside this python module, to help you with edition, although you can use your own ones.") +" "+
         _("With this mode you can create new documents"), 
-        "Puntitos"
+        "BulletsLevel1"
     )
     
     doc.addParagraph("""from unogenerator import ODT_Standard
 doc=ODT_Standard()"""    , "Code")
 
-        
     doc.addParagraph(
         _("ODT with template or file (Recomended).") + " " + 
         _("With this mode you can read your files to overwrite them or use your file as a new template to create new documents"), 
-        "Puntitos"
+        "BulletsLevel1"
     )
     
     doc.addParagraph("""from unogenerator import ODT
@@ -305,7 +316,7 @@ doc=ODT('yourdocument.odt')"""    , "Code")
         _("ODT without template.") + " " + 
         _("With this mode you can write your files with Libreoffice default styles.") +" " +
         _("If you want to create new ones, you should write them using Libreoffice API code"), 
-        "Puntitos"
+        "BulletsLevel1"
     )
     
     
@@ -319,33 +330,31 @@ doc=ODT()"""    , "Code")
     )
     doc.addParagraph("""doc.print_styles()"""    , "Code")
 
-    
-
     doc.addParagraph(_("Tables"), "Heading 2")
     table_data=[
-        [_("Concept"), _("Value") ], 
-        [_("Text"), _("This is a text")], 
-        [_("Datetime"), datetime.now()], 
-        [_("Date"), date.today()], 
-        [_("Float"),  12.121], 
-        [_("Currency"), Currency(-12.12, "EUR")], 
-        [_("Percentage"), Percentage(1, 3)], 
+        [_("Concept"), _("Value"), _("Commnet") ], 
+        [_("Text"), _("This is a text"), _("Good")], 
+        [_("Datetime"), datetime.now(), _("Good")], 
+        [_("Date"), date.today(), _("Good")], 
+        [_("Float"),  12.121, _("Good")], 
+        [_("Currency"), Currency(-12.12, "EUR"), _("Good")], 
+        [_("Percentage"), Percentage(1, 3), _("Good")], 
     ]
     
+    columnspercentages=[15, 70, 15 ]
+    doc.addParagraph(_("We can create tables with diferent font sizes and formats:") + str(columnspercentages), "Standard")
+    doc.addTableParagraph(table_data, columnssize_percentages=columnspercentages, style="Table1")
     
-    doc.addParagraph(_("We can create tables with diferent font sizes and formats:"), "Standard")
-    doc.addTableParagraph(table_data, columnssize_percentages=[30, ])
-    
-    doc.addTableParagraph(table_data, columnssize_percentages=[30, ],  size=6, style="3D")
+    doc.addTableParagraph(table_data, columnssize_percentages=[30, 40,30],  size=6, style="Table1")
     
 
     doc.addParagraph(_("Lists and numbered lists"), "Heading 2") 
-    doc.addParagraph(_("Simple list"), "Standard")
-    doc.addListPlain([
-        "Prueba hola no. Prueba hola no. Prueba hola no. Prueba hola no. Prueba hola no. Prueba hola no. Prueba hola no. ", 
-        "Adios", 
-        "Bienvenido"
-    ])       
+    doc.addParagraph(_("Simple list"), "BulletsLevel1")
+    doc.addParagraph(_("Simple list"), "BulletsLevel2")
+    doc.addParagraph(_("Simple list"), "BulletsLevel2")
+    doc.addParagraph(_("Simple list"), "BulletsLevel1")
+    doc.addParagraph(_("Simple list"), "BulletsLevel2")
+    doc.addParagraph(_("Simple list"), "BulletsLevel1")
 
 
     doc.pageBreak()
@@ -374,17 +383,24 @@ doc=ODT()"""    , "Code")
 
     doc.pageBreak()
     doc.addParagraph(_("ODS"), "Heading 1")
-    doc.addParagraph("""    TTO READ
-def demo_ods_standard_read():
-    doc=ODS("unogenerator_ods_standard.ods")
-    doc.setActiveSheet(0)
-    print(doc.getValuesByRow("4", standard=True))
-    print(doc.getValuesByRow("4", standard=False))
-    doc.close()
-    return "demo_ods_standard took {}".format(datetime.now()-doc.init)""",  "Standard")
+
+    doc.addParagraph(_("ODS 'Hello World' example"), "Heading 2")
+    doc.addParagraph(_("This is a Hello World example. You'll get the example in ods, xlsx and pdf formats:") ,  "Standard")
+    doc.addParagraph("""from unogenerator import ODS_Standard
+doc=ODS_Standard()
+doc.addCellMergedWithStyle("A1:E1", "Hello world", style="BoldCenter")
+doc.save("hello_world.ods")
+doc.export_xlsx("hello_world.xlsx")
+doc.export_pdf("hello_world.pdf")
+doc.close()"""    , "Code")
     
-    doc.find_and_replace_and_setcursorposition("%REPLACEME%")
-    doc.addParagraph(_("This paragraph was set at the end of the code after a find and replace command."), "Standard")
+    doc.find_and_replace("%REPLACEME%", _("This paragraph was set at the end of the code after a find and replace command."))
+    doc.paragraphBreak()
+    doc.addParagraph(_("This paragraph was set after replacement."), "Standard")
+    doc.pageBreak()
+    doc.addParagraph(_("This paragraph was set after a page break."), "Standard")
+    doc.pageBreak("Landscape")
+    doc.addParagraph(_("This paragraph was set after a page break with Landscape style."), "Standard")
     
     
     
@@ -392,7 +408,7 @@ def demo_ods_standard_read():
     doc.export_docx(f"unogenerator_documentation_{language}{suffix}.docx")
     doc.export_pdf(f"unogenerator_documentation_{language}{suffix}.pdf")
     doc.close()
-    r=f"unogenerator_documentation_{language}{suffix}.odt took {datetime.now()-doc.init} in {port}"
+    r=_(f"unogenerator_documentation_{language}{suffix}.odt took {datetime.now()-doc.init} in {port}")
     print(r)
     return r
 
