@@ -8,30 +8,23 @@ from com.sun.star.beans import PropertyValue
 from com.sun.star.text import ControlCharacter
 from com.sun.star.awt import Size
 from com.sun.star.style.ParagraphAdjust import RIGHT,  LEFT
-from com.sun.star.style.BreakType import PAGE_AFTER
+from com.sun.star.style.BreakType import PAGE_BEFORE, PAGE_AFTER
 from gettext import translation
-from logging import warning
+from logging import warning, debug
 from pkg_resources import resource_filename
-from unogenerator.commons import Coord as C, ColorsNamed,  Range as R, datetime2uno, guess_object_style, row2index, column2index, datetime2localc1989, date2localc1989,  time2localc1989, Coord_from_letters, Coord_from_index
+from shutil import copyfile
+from tempfile import TemporaryDirectory
+from unogenerator.commons import Coord as C, ColorsNamed,  Range as R, datetime2uno, guess_object_style, row2index, column2index, datetime2localc1989, date2localc1989,  time2localc1989, Coord_from_letters, Coord_from_index, next_port, get_from_process_numinstances_and_firstport
 from unogenerator.reusing.currency import Currency
 from unogenerator.reusing.datetime_functions import string2dtnaive, string2date, string2time
+from unogenerator.reusing.decorators import timeit
 from unogenerator.reusing.percentage import Percentage
 
 def createUnoService(serviceName):
 #        resolver = localContext.ServiceManager.createInstance('com.sun.star.bridge.UnoUrlResolver')
   return getComponentContext().ServiceManager.createInstance(serviceName)
 
-def executeDispatch(filename, self, linked):
-        ## EJEMPLO ADAPTAR CUANDO SE NECESITE
-        oDisp = createUnoService("com.sun.star.frame.DispatchHelper")
-        oProps=(
-            PropertyValue('FileName',0,f"file://{path.abspath(filename)}",0),
-            PropertyValue('AsLink',0, linked,0),
-        )
-        oDisp.executeDispatch(self.document.getCurrentController().Frame, ".uno:InsertGraphic", "", 0, oProps)
-        numberImages=self.document.getGraphicObjects().Count        
-        image=self.document.getGraphicObjects().getByIndex(numberImages-1)
-        print(image)
+        
 try:
     t=translation('unogenerator', resource_filename("unogenerator","locale"))
     _=t.gettext
@@ -41,14 +34,42 @@ except:
 
 class ODF:
     def __init__(self, template=None, loserver_port=2002):
-        self.template=None if template is None else f"file://{path.abspath(template)}"
+        self.template=None if template is None else systemPathToFileUrl(path.abspath(template))
+        self.loserver_port=loserver_port
         self.init=datetime.now()
-        
-        localContext = getComponentContext()
-        resolver = localContext.ServiceManager.createInstance('com.sun.star.bridge.UnoUrlResolver')
-        ctx = resolver.resolve(f'uno:socket,host=127.0.0.1,port={loserver_port};urp;StarOffice.ComponentContext')
-        self.desktop = ctx.ServiceManager.createInstance('com.sun.star.frame.Desktop')
-        self.graphicsprovider=ctx.ServiceManager.createInstance("com.sun.star.graphic.GraphicProvider")
+        self.num_instances, self.first_port=get_from_process_numinstances_and_firstport()
+        for i in range(self.num_instances):
+            try:
+                localContext = getComponentContext()
+                resolver = localContext.ServiceManager.createInstance('com.sun.star.bridge.UnoUrlResolver')
+                ctx = resolver.resolve(f'uno:socket,host=127.0.0.1,port={loserver_port};urp;StarOffice.ComponentContext')
+                self.desktop = ctx.ServiceManager.createInstance('com.sun.star.frame.Desktop')
+                self.graphicsprovider=ctx.ServiceManager.createInstance("com.sun.star.graphic.GraphicProvider")                   
+                args=(
+                    PropertyValue('AsTemplate',0,True,0),
+                )
+                if self.__class__  in (ODS, ODS_Standard):
+                    if self.template is None:
+                        self.document=self.desktop.loadComponentFromURL('private:factory/scalc','_blank',8,())
+                    else:
+                        self.document=self.desktop.loadComponentFromURL(self.template,'_blank', 8, args)
+                    self.sheet=self.setActiveSheet(0)
+                    self.numcells=0
+                    self.numgetvalues=0
+                else: #ODT
+                    if self.template is None:
+                        self.document=self.desktop.loadComponentFromURL('private:factory/swriter','_blank', 8, ())
+                    else:
+                        self.document=self.desktop.loadComponentFromURL(self.template,'_blank', 8, args)
+                    self.cursor=self.document.Text.createTextCursor()
+                break
+            except Exception as e:
+                print(e)
+                old=self.loserver_port
+                self.loserver_port=next_port(self.loserver_port, self.first_port, self.num_instances)
+                print(_(f"Changing port {old} to {self.loserver_port} {i} times"))
+                if i==self.num_instances-1:
+                    print(_("This process died"))
 
 
     def calculateAll(self):
@@ -82,23 +103,31 @@ class ODF:
         self.document.DocumentProperties.CreationDate=datetime2uno(creationdate)
         self.document.DocumentProperties.ModificationDate=datetime2uno(creationdate)
         self.document.DocumentProperties.Title=title
-                    
+
+    def deleteAll(self):
+        self.executeDispatch(".uno:SelectAll")
+        self.executeDispatch(".uno:Delete")
+        
+    def executeDispatch(self, command):
+        oDisp = createUnoService("com.sun.star.frame.DispatchHelper")
+        oDisp.executeDispatch(self.document.getCurrentController().Frame, command, "", 0, ())
+        
+    ## Poner el tray en el resover y cambiar el puerto cuando except
+    
+    def loadStylesFromFile(self, filename, overwrite=True):
+        styleoptions=list(self.document.StyleFamilies.StyleLoaderOptions)#it's a tuple
+        styleoptions.pop()
+        styleoptions.append(PropertyValue("OverwriteStyles",0,overwrite,0))
+        self.document.StyleFamilies.loadStylesFromURL(systemPathToFileUrl(filename), tuple(styleoptions))
+
+        
+                   
 class ODT(ODF):
     def __init__(self, template=None, loserver_port=2002):
         ODF.__init__(self, template, loserver_port)
-        
-        args=(
-            PropertyValue('AsTemplate',0,True,0),
-        )
-        if self.template is None:
-            self.document=self.desktop.loadComponentFromURL('private:factory/swriter','_blank',0,())
-        else:
-            self.document=self.desktop.loadComponentFromURL(self.template,'_blank',0,args)
-        self.cursor=self.document.Text.createTextCursor()
 
-        
+    @timeit
     def save(self, filename, overwrite_template=False):
-        filename=f"file://{path.abspath(filename)}"
         if filename==self.template and overwrite_template is False:
             print(_("You can't use the same filename as your template or you will overwrite it."))
             print(_("You can force it, setting overwrite_template paramter to True"))
@@ -113,11 +142,15 @@ class ODT(ODF):
             PropertyValue('FilterName',0,'writer8',0),
             PropertyValue('Overwrite',0,True,0),
         )
-        self.document.storeAsURL(filename, args)
+    
+
+        with TemporaryDirectory() as tmpdirname:
+                tempfile=f"{tmpdirname}/{path.basename(filename)}"
+                self.document.storeAsURL(systemPathToFileUrl(tempfile), args)
+                copyfile(tempfile, filename)
         
                 
     def export_pdf(self, filename):
-        filename=f"file://{path.abspath(filename)}"
         if filename.endswith(".pdf") is False:
             print(_("Filename extension must be 'pdf'."))
             print(_("Document hasn't been saved."))
@@ -126,25 +159,12 @@ class ODT(ODF):
             PropertyValue('FilterName',0,'writer_pdf_Export',0),
             PropertyValue('Overwrite',0,True,0),
         )
-        self.document.storeToURL(filename, args)
+        with TemporaryDirectory() as tmpdirname:
+            tempfile=f"{tmpdirname}/{path.basename(filename)}"
+            self.document.storeToURL(systemPathToFileUrl(tempfile), args)
+            copyfile(tempfile, filename)
          
-##def insertTextIntoCell( table, cellName, text, color ):
-##    tableText = table.getCellByName( cellName )
-##    cursor = tableText.createTextCursor()
-##    cursor.setPropertyValue( "CharColor", color )
-##    tableText.setString( text )
-    def find_and_replace_and_setcursorposition(self, find, replace=""):
-        search=self.document.createSearchDescriptor()
-        search.SearchString=find
-        found=self.document.findFirst(search)
-        if found is not None:
-            found.setString(replace)
-            self.cursor=found
-        else:
-            warning(f"'{find}' was not found in the document'")
-
     def export_docx(self, filename):
-        filename=f"file://{path.abspath(filename)}"
         if filename.endswith(".docx") is False:
             print(_("Filename extension must be 'docx'."))
             print(_("Document hasn't been saved."))
@@ -153,7 +173,27 @@ class ODT(ODF):
             PropertyValue('FilterName',0,'MS Word 2007 XML',0),
             PropertyValue('Overwrite',0,True,0),
         )
-        self.document.storeToURL(filename, args)
+
+        with TemporaryDirectory() as tmpdirname:
+            tempfile=f"{tmpdirname}/{path.basename(filename)}"
+            self.document.storeToURL(systemPathToFileUrl(tempfile), args)
+            copyfile(tempfile, filename)
+##def insertTextIntoCell( table, cellName, text, color ):
+##    tableText = table.getCellByName( cellName )
+##    cursor = tableText.createTextCursor()
+##    cursor.setPropertyValue( "CharColor", color )
+##    tableText.setString( text )
+    def find_and_replace(self, find, replace=""):
+        search=self.document.createSearchDescriptor()
+        search.SearchString=find
+        found=self.document.findFirst(search)
+        if found is not None:
+            found.setString("")
+            self.cursor=found
+            self.document.Text.insertString(self.cursor, replace, False)
+        else:
+            warning(f"'{find}' was not found in the document'")
+
         
     def addParagraph(self,  text,  style):
         self.cursor.setPropertyValue("ParaStyleName", style)
@@ -198,15 +238,16 @@ class ODT(ODF):
 
     ## Table data, must be all strings (Not a spreadsheet)
     ## @params magings TRBL
-    ## @params columnssize_percentage Por ejemplo para table 30,70, solo debo poner [30,]
+    ## @params columnssize_percentage Por ejemplo para table 30,70, solo debo poner [30,70]
     ##   para table [10,10,80], solo debo poner [10,10]
     ## if none , rangos iguales
     def addTableParagraph(self, 
         data,  
         columnssize_percentages=None, 
-        margins_top_bottom=500,  
+        margins_top=0.3,
+        margins_bottom=0.3,
         size=10,  
-        width_percentage=80, 
+        width_percentage=100, 
         alignment="center",  
         paragraph_style="Standard", 
         style=None, 
@@ -225,7 +266,6 @@ class ODT(ODF):
         #Párrafo de la table
         self.cursor.setPropertyValue("ParaStyleName", paragraph_style)
         self.document.Text.insertTextContent(self.cursor,table,False)
-        self.document.Text.insertControlCharacter(self.cursor, ControlCharacter.PARAGRAPH_BREAK, False)
         
         #Table data, must be all strings (Not a spreadsheet)
         if style is not None:
@@ -237,29 +277,41 @@ class ODT(ODF):
                 cursor.CharHeight=size
                 if str(cell_data).startswith("-"):
                     cursor.setPropertyValue( "CharColor", 0xDD0000 )
-                if cell_data.__class__.__name__ in ["str"]:
-                    cursor.ParaAdjust=LEFT
+                if style in ["Table0", "Table1", "Table1Total"] and row==0:
+                    pass # Centered text for styles headers
                 else:
-                    cursor.ParaAdjust=RIGHT
-                    cursor.ParaRightMargin=100
+                    if cell_data.__class__.__name__ in ["str"]:
+                        cursor.ParaAdjust=LEFT
+                    else:
+                        cursor.ParaAdjust=RIGHT
+                        cursor.ParaRightMargin=100
                 cell.insertString(cursor, str(cell_data), False)
         
         #TAble width and style
         table.HoriOrient=2 #Centered
-        table.TopMargin=margins_top_bottom
-        table.BottomMargin=margins_top_bottom
+        table.TopMargin=margins_top*1000
+        table.BottomMargin=margins_bottom*1000
         table.RelativeWidth=width_percentage #PARECE QUE ES SOLO DESCRIPTIVO
-        
+        #print("RelativeSum", table.TableColumnRelativeSum)
         #Columns width
         if columnssize_percentages is not None:
             separators=[]
-            for sep in columnssize_percentages:
+            #print("Before", *self.printSeparators(table, table.TableColumnSeparators))
+            for i, sep in enumerate(columnssize_percentages[:-1]):
                 separator= createUnoStruct("com.sun.star.text.TableColumnSeparator")
-                separator.Position=sep*100
+                separator.Position=sum(columnssize_percentages[0:i+1])*100
                 separator.IsVisible=True
                 separators.append(separator)
-            #        print(table.TableColumnRelativeSum)
+            #print("Setting",  *self.printSeparators(table, separators))
             table.TableColumnSeparators=separators
+            #print("After", *self.printSeparators(table, table.TableColumnSeparators))
+            #print()
+
+    def printSeparators(self, table, separators):
+            r=[]
+            for i in range(len(separators)):
+                r.append(separators[i].Position)
+            return (table.TableColumnRelativeSum, str(r))
         
     def addListPlain(self, arr, list_style="List_2", paragraph_style="Puntitos"):
 #        def get_items(list_o, list_style, paragraph_style):
@@ -280,37 +332,27 @@ class ODT(ODF):
 #        # #########################
         for s in arr:
             self.addParagraph(s, paragraph_style)
+            
+    def paragraphBreak(self):
+        self.document.Text.insertControlCharacter(self.cursor, ControlCharacter.PARAGRAPH_BREAK, False)
         
-    def pageBreak(self, style="Standard"):
-        self.cursor.setPropertyValue("ParaStyleName", style)
+    ## Default styles are Landscape
+    def pageBreak(self, style="Standard", page_before=True):
+        if page_before is True:
+            self.cursor.BreakType = PAGE_BEFORE 
+        else:
+            self.cursor.BreakType = PAGE_AFTER
+            
+        self.cursor.setPropertyValue("PageDescName", style)
         self.document.Text.insertString(self.cursor, "", False)
-        self.cursor.BreakType = PAGE_AFTER
-        self.document.Text.insertControlCharacter(self.cursor.End, ControlCharacter.PARAGRAPH_BREAK, False)
-
 class ODS(ODF):
+    @timeit
     def __init__(self, template=None, loserver_port=2002):
         ODF.__init__(self, template, loserver_port)
         
-        args=(
-            PropertyValue('AsTemplate',0,True,0),
-        )
-        self.document=None
-        if self.template is None:
-            self.document=self.desktop.loadComponentFromURL('private:factory/scalc','_blank',8,())
-        else:
-            self.document=self.desktop.loadComponentFromURL(self.template,'_blank',8   , args)
-#        for i in range(1000):
-#            sleep(0.1)
-#            if i%100==0:
-#                print("Document is none, retrying", i,  loserver_port,  self.desktop.Frames.Count)
-#            if self.document is not None:
-#                break
-
-        self.sheet_index=0
-        self.sheet=self.setActiveSheet(self.sheet_index)
-            
     ## Creates a new sheet at the end of the sheets
     ## @param if index is None it creates sheet at the end of the existing sheets
+    @timeit
     def createSheet(self, name, index=None):
         sheets=self.document.getSheets()
         if index is None:
@@ -318,6 +360,7 @@ class ODS(ODF):
         sheets.insertNewByName(name, index)
         self.setActiveSheet(index)
         
+    @timeit
     def removeSheet(self, index):
         current=self.sheet.Name
         self.setActiveSheet(index)
@@ -327,8 +370,10 @@ class ODS(ODF):
     def setActiveSheet(self,  index):
         self.sheet_index=index
         self.sheet=self.document.getSheets().getByIndex(index)
+        return self.sheet
     
     ## l measures are in cm can be float
+    @timeit
     def setColumnsWidth(self, l):
         columns=self.sheet.getColumns()
         for i, width in enumerate(l):
@@ -404,6 +449,7 @@ class ODS(ODF):
             self.addColumnWithStyle(coord_start.addColumnCopy(i), column, colors=colors,styles=styles)
 
     ## @param style If None tries to guess it
+    @timeit
     def addCellWithStyle(self, coord, o, color=ColorsNamed.White, style=None):
         coord=C.assertCoord(coord)
         if style is None:
@@ -412,6 +458,7 @@ class ODS(ODF):
         self.__object_to_cell(cell, o)
         cell.setPropertyValue("CellStyle", style)
         cell.setPropertyValue("CellBackColor", color)
+        self.numcells=self.numcells+1
         
     ## All of this names are document names
     def setCellName(self, coord, name):
@@ -446,25 +493,36 @@ class ODS(ODF):
         else:
             cell.setString(str(o))
             print("MISSING", o.__class__.__name__)
-        
+
+    def showStatistics(self):
+        debug(f"- Number of cells {self.numcells}")
+        debug(f"- Number of get values {self.numgetvalues}")
+        debug(f"- Total time {datetime.now()-self.init}")
+
+    @timeit
     def addCellMergedWithStyle(self, range, o, color=ColorsNamed.White, style=None):
         range=R.assertRange(range)
         cell=self.sheet.getCellByPosition(range.start.letterIndex(), range.start.numberIndex())
         cellrange=self.sheet.getCellRangeByName(range.string())
         cellrange.merge(True)
         self.__object_to_cell(cell, o)
+        if style is None:
+            style=guess_object_style(o)
         cell.setPropertyValue("CellStyle", style)
         cell.setPropertyValue("CellBackColor", color)
+        self.numcells=self.numcells+1
 
+    @timeit
     def freezeAndSelect(self, freeze, selected=None, topleft=None):
         freeze=C.assertCoord(freeze) 
-        selected=None if selected is None else C.assertCoord(selected)
-        topleft=None if topleft is None else C.assertCoord(topleft)
+        num_columns, num_rows=self.getSheetSize()
         self.document.getCurrentController().setActiveSheet(self.sheet)
         self.document.getCurrentController().freezeAtPosition(freeze.letterIndex(), freeze.numberIndex())
 
         if selected is None:
-            selected=Coord_from_index(self.columnNumber()-1, self.rowNumber()-1)
+            selected=Coord_from_index(num_columns-1, num_rows-1)
+        else:
+            selected=C.assertCoord(selected)
         selectedcell=self.sheet.getCellByPosition(selected.letterIndex(), selected.numberIndex())
         self.document.getCurrentController().select(selectedcell)
         
@@ -480,13 +538,17 @@ class ODS(ODF):
             else:
                 number=minus_coord.number
             topleft=Coord_from_letters(letter, number)
+        else:
+            topleft=C.assertCoord(topleft)
         self.document.getCurrentController().setFirstVisibleColumn(topleft.letterIndex())
         self.document.getCurrentController().setFirstVisibleRow(topleft.numberIndex())
         
     def getValue(self, coord, standard=True):
         coord=C.assertCoord(coord)
+        self.numgetvalues=self.numgetvalues+1
         return self.getValueByPosition(coord.letterIndex(), coord.numberIndex(), standard)
         
+    @timeit
     def getValueByPosition(self, letter_index, number_index, standard=True):
         return self.__cell_to_object(self.sheet.getCellByPosition(letter_index, number_index), standard)
 
@@ -564,19 +626,16 @@ class ODS(ODF):
         endcoord=C("A1").addRow(self.rowNumber()-1).addColumn(self.columnNumber()-1)
         return R("A1:" + endcoord.string())
 
-    ## Esta función puede que sea costosa
-    def rowNumber(self):
-        return len(self.sheet.getData())
-        
-    ## Esta función puede que sea costosa
-    def columnNumber(self):
-        if len(self.sheet.getData())==0:
-            return 0
+    ## Returns (columnsNumber, rowsNumber
+    @timeit
+    def getSheetSize(self):
+        data=self.sheet.getData()
+        if len(data)==0:
+            return 0, 0
         else:
-            return len(self.sheet.getData()[0])
+            return len(data[0]), len(data)
             
     def save(self, filename, overwrite_template=False):
-        filename=f"file://{path.abspath(filename)}"
         if filename==self.template and overwrite_template is False:
             print(_("You can't use the same filename as your template or you will overwrite it."))
             print(_("You can force it, setting overwrite_template paramter to True"))
@@ -591,10 +650,13 @@ class ODS(ODF):
             PropertyValue('FilterName', 0, 'calc8', 0),
             PropertyValue('Overwrite', 0, True, 0),
         )
-        self.document.storeAsURL(filename, args)
+        with TemporaryDirectory() as tmpdirname:
+                tempfile=f"{tmpdirname}/{path.basename(filename)}"
+                self.document.storeAsURL(systemPathToFileUrl(tempfile), args)
+                copyfile(tempfile, filename)
+        self.showStatistics()
 
     def export_pdf(self, filename):
-        filename=f"file://{path.abspath(filename)}"
         if filename.endswith(".pdf") is False:
             print(_("Filename extension must be 'pdf'."))
             print(_("Document hasn't been saved."))
@@ -603,10 +665,13 @@ class ODS(ODF):
             PropertyValue('FilterName',0,'calc_pdf_Export',0),
             PropertyValue('Overwrite',0,True,0),
         )
-        self.document.storeToURL(filename, args)
+        with TemporaryDirectory() as tmpdirname:
+            tempfile=f"{tmpdirname}/{path.basename(filename)}"
+            self.document.storeToURL(systemPathToFileUrl(tempfile), args)
+            copyfile(tempfile, filename)
+
 
     def export_xlsx(self, filename):
-        filename=f"file://{path.abspath(filename)}"
         if filename.endswith(".xlsx") is False:
             print(_("Filename extension must be 'xlsx'."))
             print(_("Document hasn't been saved."))
@@ -615,7 +680,11 @@ class ODS(ODF):
             PropertyValue('FilterName',0,'Calc MS Excel 2007 XML',0),
             PropertyValue('Overwrite',0,True,0),
         )
-        self.document.storeToURL(filename, args)
+        with TemporaryDirectory() as tmpdirname:
+            tempfile=f"{tmpdirname}/{path.basename(filename)}"
+            self.document.storeToURL(systemPathToFileUrl(tempfile), args)
+            copyfile(tempfile, filename)
+
 
 class ODS_Standard(ODS):
     def __init__(self, loserver_port=2002):
@@ -624,3 +693,6 @@ class ODS_Standard(ODS):
 class ODT_Standard(ODT):
     def __init__(self, loserver_port=2002):
         ODT.__init__(self, resource_filename(__name__, 'templates/standard.odt'), loserver_port)
+        self.deleteAll()
+
+
